/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app_debug.h>
#include <Eina.h>
#include <media_content.h>
#include <app_contents.h>

#include "data_photo.h"
#include "datamgr.h"
#include "defs.h"

#define MEDIA_CONDITION_IMAGE "MEDIA_TYPE=0"

static bool _get_media(media_info_h m, void *data)
{
	int r;
	char *thumb, *mediaid;
	Eina_List **list;
	struct datamgr_item *di;
	bool favorite;

	if (!m) {
		_ERR("Invalid argument");
		return false;
	}

	list = data;

	r = media_info_get_media_id(m, &mediaid);
	if (r != MEDIA_CONTENT_ERROR_NONE || !mediaid) {
		_ERR("failed to get media id");
		return false;
	}

	r = media_info_get_thumbnail_path(m, &thumb);
	if (r != MEDIA_CONTENT_ERROR_NONE || !thumb) {
		_ERR("failed to get thumb");
		free(mediaid);
		return true;
	}

	favorite = false;
	r = app_contents_favorite_check(CONTENTS_PHOTO, mediaid, &favorite);
	if (r != APP_CONTENTS_ERROR_NONE)
		_ERR("failed to get favorite");

	di = calloc(1, sizeof(*di));
	if (!di) {
		_ERR("failed to calloc datamgr item");
		free(mediaid);
		free(thumb);
		return false;
	}
	di->icon = strdup(thumb);
	if (favorite)
		di->focus_icon = strdup(IMAGE_PHOTO_FAVORITE);

	*list = eina_list_append(*list, di);

	free(mediaid);
	free(thumb);

	return true;
}

static void _unload_photo(struct datamgr *dm)
{
	struct datamgr_item *di;

	EINA_LIST_FREE(dm->list, di) {
		free(di->icon);
		free(di->focus_icon);

		free(di);
	}

	dm->list = NULL;
}

static Eina_List *_get_items(struct datamgr *dm)
{
	filter_h f;
	int r;

	if (!dm) {
		_ERR("Invalid argument");
		return NULL;
	}

	_unload_photo(dm);

	r = media_filter_create(&f);
	if (r != MEDIA_CONTENT_ERROR_NONE) {
		_ERR("failed to created filter");
		return NULL;
	}

	media_filter_set_condition(f, MEDIA_CONDITION_IMAGE,
			MEDIA_CONTENT_COLLATE_DEFAULT);

	r = media_info_foreach_media_from_db(f,
			_get_media, &dm->list);
	if (r != MEDIA_CONTENT_ERROR_NONE)
		_ERR("folder foreach failed");

	media_filter_destroy(f);


	return dm->list;
}

static bool _init(struct datamgr *dm)
{
	return true;
}

static void _fini(struct datamgr *dm)
{
	if (!dm) {
		_ERR("Invalid argument");
		return;
	}

	_unload_photo(dm);
}

static struct data_class dclass = {
	.init = _init,
	.fini = _fini,
	.get_items = _get_items
};

struct data_class *datamgr_photo_get_dclass(void)
{
	return &dclass;
}
